/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useEffect, useState } from 'react';
import {
  AppBar,
  Toolbar,
  CssBaseline,
  Typography,
  MenuItem,
  ListItemText,
  Menu,
  IconButton,
  Button,
} from '@mui/material';
import { AccountCircle } from '@mui/icons-material';
import logo from './logogp.png';
import logo_white from './logogpwhite.png';
import AddDatabaseForm from './AddDatabaseForm';
import { useTheme } from '@mui/material/styles';
import styles from './navbar.module.scss';

import {
  updateCurrentDatabase,
  updateDatabaseList,
} from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import {
  AddDatabaseRequest,
  useAppDispatch,
  useAuthorizationCache,
  useDatabaseAPI,
  useSchemaAPI,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { domain } from '../../environments/variables';

/** NavbarComponentProps is an interface containing the NavbarViewModel. */
export interface NavbarComponentProps {
  // changeColourPalette: () => void; FIXME move to redux
}

/** NavbarComponentState is an interface containing the type of visualizations. */
export interface NavbarComponentState {
  // The anchor for rendering the user data (clientID, sessionID, add database, etc) menu
  userMenuAnchor?: Element;
  // The anchor for rendering the menu for selecting a database to use
  selectDatabaseMenuAnchor?: Element;
  deleteDatabaseMenuAnchor?: Element;
  // Determines if the addDatabaseForm will be shown
  showAddDatabaseForm: boolean;
}

export interface NavbarSubComponentState {
  changeDb?: String;
  deleteDb?: String;
}

/** NavbarComponent is the View implementation for Navbar */
export const Navbar = (props: NavbarComponentProps) => {
  const theme = useTheme();
  const auth = useAuthorizationCache();
  const session = useSessionCache();
  const api = useDatabaseAPI(domain);
  const schemaApi = useSchemaAPI(domain);
  const dispatch = useAppDispatch();

  useEffect(() => {
    console.log(auth);
  }, [auth.accessToken]);

  // const { navbarViewModel, currentColours } = props;
  // this.navbarViewModel = navbarViewModel;

  const [state, setState] = useState<NavbarComponentState>({
    userMenuAnchor: undefined,
    selectDatabaseMenuAnchor: undefined,
    deleteDatabaseMenuAnchor: undefined,
    showAddDatabaseForm: false,
  });

  const [subMenuChangeDisplay, setSubMenuChangewDisplay] = useState('block');
  const [subMenuDeleteDisplay, setSubMenuDeletewDisplay] = useState('block');

  /** Closes the user menu. */
  function closeUserMenu(): void {
    // If a nested window is open, close the main user menu a bit later
    setState({
      ...state,
      selectDatabaseMenuAnchor: undefined,
      deleteDatabaseMenuAnchor: undefined,
    });

    setTimeout(() => {
      setState({ ...state, userMenuAnchor: undefined });
    }, 100);
  }

  /** Open nested menu's. */
  const openChangeSubMenu = () => {
    if (subMenuChangeDisplay === 'none') {
      setSubMenuChangewDisplay('block');
    }
  };
  const openDeleteSubMenu = () => {
    if (subMenuDeleteDisplay === 'none') {
      setSubMenuDeletewDisplay('block');
    }
  };

  /** Closes nested menu's. */
  const closeChangeSubMenu = () => {
    setSubMenuChangewDisplay('none');
  };
  const closeDeleteSubMenu = () => {
    setSubMenuDeletewDisplay('none');
  };

  function changeColourPalette() {
    // TODO
  }

  /**
   * Called when the user clicks on the 'submit' button of the add database form.
   */
  function onAddDatabaseFormSubmit(
    request: AddDatabaseRequest
  ): Promise<void | Response> {
    return api
      .AddDatabase(request, { updateDatabaseCache: true, setAsCurrent: true })
      .then(() => {
        schemaApi.RequestSchema(request.name);
      });
  }

  const currentLogo = theme.palette.custom.logo == 'white' ? logo_white : logo;

  return (
    <div className={styles.root}>
      <CssBaseline />
      <AppBar
        title="GraphPolaris"
        style={{
          backgroundColor: theme.palette.custom.background,
        }}
        sx={{
          zIndex: 9999,
        }}
        position="fixed"
        className={styles.appBar}
      >
        <Toolbar style={{ marginLeft: -5 }}>
          <a href="https://graphpolaris.com/" className={styles.logo}>
            <img src={currentLogo} className={styles.logo} />
          </a>
          <div className={styles.menubox}>
            {/* <Button
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
              onClick={changeColourPalette}
            >
              Change Palette
            </Button> */}
            <Button
              href="https://graphpolaris.com/"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Home
            </Button>
            <Button
              href="https://graphpolaris.com/index.php/products/"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Products
            </Button>
            <Button
              href="https://graphpolaris.com#usecases"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Use Cases
            </Button>
            <Button
              href="https://graphpolaris.com#earlyadoption"
              className={styles.menuText}
              style={{ color: theme.palette.custom.menuText }}
            >
              Contact
            </Button>
            {auth.authorized ? (
              <div>
                <IconButton
                  color="inherit"
                  onClick={(event) =>
                    setState({
                      ...state,
                      userMenuAnchor: event.currentTarget,
                    })
                  }
                >
                  <AccountCircle htmlColor={theme.palette.custom.menuText} />
                </IconButton>
                <Menu
                  id="user-menus"
                  anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
                  transformOrigin={{ vertical: 'top', horizontal: 'left' }}
                  anchorEl={state.userMenuAnchor}
                  keepMounted
                  open={Boolean(state.userMenuAnchor)}
                  onClose={() => closeUserMenu()}
                >
                  <MenuItem>
                    <ListItemText primary={'userID: ' + auth.userID} />
                  </MenuItem>
                  <MenuItem>
                    <ListItemText primary={'sessionID: ' + auth.sessionID} />
                  </MenuItem>
                  {/* <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('google')
                    }
                  >
                    <ListItemText primary={'request new'} />
                  </MenuItem> */}
                  {/*<MenuItem onClick={() => LocalStorage.instance().Reset()}>*/}
                  {/*  <ListItemText primary={'Empty localstorage'} />*/}
                  {/*</MenuItem>*/}
                  <MenuItem
                    onClick={() =>
                      setState({
                        ...state,
                        userMenuAnchor: undefined,
                        showAddDatabaseForm: true,
                      })
                    }
                  >
                    <ListItemText primary={'Add database'} />
                  </MenuItem>
                  <MenuItem
                    onClick={(event) =>
                      setState({
                        ...state,
                        selectDatabaseMenuAnchor: event.currentTarget,
                      })
                    }
                  >
                    <ListItemText
                      onClick={() => openChangeSubMenu()}
                      primary={'Change database'}
                    />
                  </MenuItem>
                  <Menu
                    id="databases-menus"
                    anchorOrigin={{ vertical: 'top', horizontal: 'left' }}
                    transformOrigin={{ vertical: 'top', horizontal: 'right' }}
                    anchorEl={state.selectDatabaseMenuAnchor}
                    keepMounted
                    disableAutoFocusItem
                    open={Boolean(state.selectDatabaseMenuAnchor)}
                    sx={{ display: subMenuChangeDisplay }}
                    onClose={() => closeChangeSubMenu()}
                  >
                    {session.databases.length > 0 ? (
                      session.databases.map((database) => (
                        <MenuItem
                          key={database}
                          selected={database == session.currentDatabase}
                          sx={{ display: subMenuChangeDisplay }}
                          onClick={() => {
                            if (session.currentDatabase != database) {
                              dispatch(updateCurrentDatabase(database));
                            }
                            closeChangeSubMenu();
                          }}
                        >
                          <ListItemText primary={database} />
                        </MenuItem>
                      ))
                    ) : (
                      <MenuItem key="placeholder" value="" disabled>
                        no databases connected
                      </MenuItem>
                    )}
                  </Menu>
                  <MenuItem
                    onClick={(event) =>
                      setState({
                        ...state,
                        deleteDatabaseMenuAnchor: event.currentTarget,
                      })
                    }
                  >
                    <ListItemText
                      onClick={() => openDeleteSubMenu()}
                      primary={'Delete database'}
                    />
                  </MenuItem>
                  <Menu
                    id="delete-databases-menus"
                    anchorOrigin={{ vertical: 'top', horizontal: 'left' }}
                    transformOrigin={{ vertical: 'top', horizontal: 'right' }}
                    anchorEl={state.deleteDatabaseMenuAnchor}
                    keepMounted
                    disableAutoFocusItem
                    open={Boolean(state?.deleteDatabaseMenuAnchor)}
                    sx={{ display: subMenuDeleteDisplay }}
                    onClose={() => closeDeleteSubMenu()}
                  >
                    {session.databases.length > 0 ? (
                      session.databases.map((database) => (
                        <MenuItem
                          key={database}
                          selected={database == session.currentDatabase}
                          onClick={() => {
                            if (session.currentDatabase === database) {
                              dispatch(updateCurrentDatabase(''));
                            }
                            api.DeleteDatabase(database);
                            closeDeleteSubMenu();
                          }}
                        >
                          <ListItemText primary={database} />
                        </MenuItem>
                      ))
                    ) : (
                      <MenuItem key="placeholder" value="" disabled>
                        no databases connected
                      </MenuItem>
                    )}
                  </Menu>
                </Menu>
              </div>
            ) : (
              <div>
                <Button
                  className={styles.loginButton}
                  onClick={(event) =>
                    setState({
                      ...state,
                      userMenuAnchor: event.currentTarget,
                    })
                  }
                >
                  <span
                    className={styles.loginButtonText}
                    style={{
                      color: theme.palette.custom.menuText,
                    }}
                  >
                    Login
                  </span>
                </Button>
                {/* <Menu
                  id="user-menus"
                  getContentAnchorEl={null}
                  anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
                  transformOrigin={{ vertical: 'top', horizontal: 'left' }}
                  anchorEl={state.userMenuAnchor}
                  keepMounted
                  open={Boolean(state.userMenuAnchor)}
                  onClose={() => closeUserMenu()}
                >
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('google')
                    }
                  >
                    <ListItemText primary={'Login with Google'} />
                  </MenuItem>
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('github')
                    }
                  >
                    <ListItemText primary={'Login with GitHub'} />
                  </MenuItem>
                  <MenuItem
                    onClick={() =>
                      navbarViewModel.handleLoginButtonClicked('free')
                    }
                  >
                    <ListItemText primary={'Login with free (debug)'} />
                  </MenuItem>
                </Menu> */}
              </div>
            )}
          </div>
        </Toolbar>
      </AppBar>
      <AddDatabaseForm
        open={state.showAddDatabaseForm}
        onClose={() => setState({ ...state, showAddDatabaseForm: false })}
        onSubmit={(...params) => {
          onAddDatabaseFormSubmit(...params);
          setState({ ...state, showAddDatabaseForm: false });
        }}
        styles={undefined}
      />
    </div>
  );
};
