import { configureStore } from '@reduxjs/toolkit';
import colorPaletteConfigSlice from './colorPaletteConfigSlice';
import graphQueryResultSlice from './graphQueryResultSlice';
import querybuilderSlice from './querybuilderSlice';
import schemaSlice from './schemaSlice';
import configSlice from './configSlice';
import sessionSlice from './sessionSlice';
import authSlice from './authSlice';

export const store = configureStore({
  reducer: {
    graphQueryResult: graphQueryResultSlice,
    schema: schemaSlice,
    colorPaletteConfig: colorPaletteConfigSlice,
    querybuilder: querybuilderSlice,
    sessionCache: sessionSlice,
    auth: authSlice,
    config: configSlice,
  },
  middleware: (getDefaultMiddleware) =>
    getDefaultMiddleware({
      serializableCheck: {
        ignoredPaths: [
          'schema.graphologySerialized',
          'querybuilder.graphologySerialized',
        ],
      },
    }),
});

// Infer the `RootState` and `AppDispatch` types from the store itself
export type RootState = ReturnType<typeof store.getState>;
// Inferred type: {counter: CounterState}
export type AppDispatch = typeof store.dispatch;
