import React from 'react';
import { Meta, StoryObj } from '@storybook/react';
import RelationPill from './relationpill';
import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';
import { GraphPolarisThemeProvider } from '@graphpolaris/shared/lib/data-access/theme';

import {
  colorPaletteConfigSlice,
  querybuilderSlice,
  schemaSlice,
} from '@graphpolaris/shared/lib/data-access/store';
import { ReactFlowProvider } from 'reactflow';
import { RelationData } from '../../../graph/reactflow/model';

const Component: Meta<typeof RelationPill> = {
  /* 👇 The title prop is optional.
   * See https://storybook.js.org/docs/react/configure/overview#configure-story-loading
   * to learn how to generate automatic titles
   */
  title: 'Querybuilder/Pills/RelationPill',
  component: RelationPill,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <GraphPolarisThemeProvider>
          <ReactFlowProvider>{story()}</ReactFlowProvider>
        </GraphPolarisThemeProvider>
      </Provider>
    ),
  ],
};

export default Component;

// A super-simple mock of a redux store
const Mockstore = configureStore({
  reducer: {
    colorPaletteConfig: colorPaletteConfigSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
    // schema: schemaSlice.reducer,
  },
});

// const Template = (args: any) => <EntityRFPill {...args} />;

export const Default: StoryObj<{ data: RelationData }> = {
  args: {
    data: {
      name: 'TestEntity',
      collection: 'test',
      depth: { min: 0, max: 1 },
      fadeIn: false,
    },
  },
};

// Default.decorators = [
//   (story) => (
//     <Provider store={Mockstore}>
//       <GraphPolarisThemeProvider>{story()}</GraphPolarisThemeProvider>
//     </Provider>
//   ),
// ];
