import {
  AlgorithmToLayoutProvider,
  AllLayoutAlgorithms,
  LayoutFactory,
} from '@graphpolaris/shared/lib/graph-layout';
import {
  schemaGraphology2Reactflow,
  schemaExpandRelation,
} from '@graphpolaris/shared/lib/schema/schema-utils';
import {
  useSchemaGraph,
  useSchemaGraphology,
  useSchemaLayout,
} from '@graphpolaris/shared/lib/data-access/store';
import { MultiGraph } from 'graphology';
// import { AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/graph-layout';
import { useEffect, useMemo, useRef, useState } from 'react';
import ReactFlow, {
  ControlButton,
  Controls,
  Node,
  Edge,
  ReactFlowProvider,
  useNodesState,
  useEdgesState,
  ReactFlowInstance,
  Background,
} from 'reactflow';

import 'reactflow/dist/style.css';

import styles from './schema.module.scss';

import {
  EntityFlowElement,
  RelationPill,
  AttributePill,
  ConnectionDragLine,
  ConnectionLine,
} from '@graphpolaris/shared/lib/querybuilder/pills';
import { EntityNode } from '../pills/nodes/entity/entity-node';
import { RelationNode } from '../pills/nodes/relation/relation-node';
import { NodeQualityEntityPopupNode } from '../pills/nodes/popup/node-quality-entity-popup';
import { NodeQualityRelationPopupNode } from '../pills/nodes/popup/node-quality-relation-popup';
import { AttributeAnalyticsPopupMenu } from '../pills/nodes/popup/attribute-analytics-popup-menu';
import NodeEdge from '../pills/edges/node-edge';
import SelfEdge from '../pills/edges/self-edge';

interface Props {
  content?: string;
}

const onInit = (reactFlowInstance: ReactFlowInstance) => {
  setTimeout(() => reactFlowInstance.fitView(), 100);
};

const nodeTypes = {
  entity: EntityNode,
  relation: RelationNode,
  // nodeQualityEntityPopup: NodeQualityEntityPopupNode,
  // nodeQualityRelationPopup: NodeQualityRelationPopupNode,
  // attributeAnalyticsPopupMenu: AttributeAnalyticsPopupMenu,

  // entity: EntityRFPill,
  // relation: RelationRFPill,
  // attribute: AttributeRFPill,
};
const edgeTypes = {
  nodeEdge: NodeEdge,
  selfEdge: SelfEdge,

  connection: ConnectionLine,
};

export const Schema = (props: Props) => {
  const [nodes, setNodes, onNodeChanged] = useNodesState([] as Node[]);
  const [edges, setEdges, onEdgeChanged] = useEdgesState([] as Edge[]);
  // In case the schema is updated
  const schemaGraphology = useSchemaGraphology();
  const schemaGraph = useSchemaGraph();
  // const [schemaGraphology, setSchema] = useState(useSchema());
  const schemaLayout = useSchemaLayout();
  const layout = useRef<AlgorithmToLayoutProvider<AllLayoutAlgorithms>>();

  // console.log('dbSchema', schemaGraphology.edges());
  // useEffect(() => {
  //   console.log('dbSchema', schemaGraphology, schemaGraphology.order);
  // }, [schemaGraphology]);

  const toggleNodeQualityPopup = (id: string) => { };
  const toggleAttributeAnalyticsPopupMenu = (id: string) => { };

  function updateLayout() {
    const layoutFactory = new LayoutFactory();
    layout.current = layoutFactory.createLayout(
      schemaLayout as AllLayoutAlgorithms
    ); // TODO: more layouts here
  }

  useEffect(() => {
    updateLayout();
  }, []);

  useEffect(() => {
    if (schemaGraphology == undefined || schemaGraphology.order == 0) {
      return;
    }
    // console.log(schemaGraphology.export());

    console.log(schemaLayout);

    updateLayout();
    const expandedSchema = schemaExpandRelation(schemaGraphology);
    layout.current?.layout(expandedSchema);

    const schemaFlow = schemaGraphology2Reactflow(expandedSchema);

    // schemaFlow.nodes.forEach((n) => {
    //   n.data.toggleNodeQualityPopup = toggleNodeQualityPopup;
    //   n.data.toggleAttributeAnalyticsPopupMenu =
    //     toggleAttributeAnalyticsPopupMenu;
    // });
    // console.log(edges);

    // console.log(
    //   'schema Layout',
    //   schemaLayout,
    //   'order',
    //   expandedSchema,
    //   schemaFlow
    // );
    setNodes(schemaFlow.nodes);
    setEdges(schemaFlow.edges);
    // console.log(
    //   'update schema useEffect',
    //   schemaGraphology,
    //   schemaGraphology.order,
    //   schemaFlow
    // );
  }, [schemaGraph, schemaLayout]);

  const graphStyles = { width: '100%', height: '500px' };

  // console.log(nodes, edges);

  return (
    <div
      style={{
        width: '100%',
        height: '100%',
      }}
    >
      {nodes.length === 0 && <p>DEBUG: No Elements</p>}
      <ReactFlowProvider>
        <ReactFlow
          className={styles.schemaPanel}
          onlyRenderVisibleElements={false}
          nodesDraggable={false}
          nodeTypes={nodeTypes}
          edgeTypes={edgeTypes}
          connectionLineComponent={ConnectionDragLine}
          onNodesChange={onNodeChanged}
          onEdgesChange={onEdgeChanged}
          nodes={nodes}
          edges={edges}
          style={graphStyles}
          onInit={onInit}
          panOnDrag={false}
          attributionPosition="top-right"
        >
          <Controls
            showInteractive={false}
            showZoom={false}
            showFitView={true}
            className={styles.controls}
          >
            {/* <ControlButton
              className={styles.exportButton}
              title={'Export graph schema'}
              onClick={(event) => {
                event.stopPropagation();
                // this.setState({
                //   ...this.state,
                //   exportMenuAnchor: event.currentTarget,
                // });
              }}
            >
              <img src={exportIcon} width={21}></img>
            </ControlButton> */}
          </Controls>
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default Schema;

// Fix layout of the schema
// create reactflow elements on xy coords
// connect reactflow elements together

// maybe ook gelijk instellingen knoppie fixen op alle panels
